import ipaddress
from typing import Optional

from orchestrator import workflow
from orchestrator.db import (
    ProductTable,
    SubscriptionTable,
    SubscriptionInstanceValueTable,
    SubscriptionInstanceTable,
    ResourceTypeTable,
)
from orchestrator.domain.base import S
from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, UUIDstr, SubscriptionLifecycle
from orchestrator.workflow import StepList, init, step, done
from orchestrator.workflows.steps import store_process_subscription, set_status, resync

from gso.products import Site
from gso.products.product_blocks import router as router_pb
from gso.products.product_blocks.router import RouterVendor, RouterRole
from gso.products.product_types import router
from gso.products.product_types.router import RouterInactive
from gso.services.crm import get_customer_by_name


@step("Create subscription")
def create_subscription(customer: str) -> State:
    customer_id: UUIDstr = get_customer_by_name(customer)["id"]
    product_id: UUIDstr = ProductTable.query.filter_by(name="Router").first().product_id
    subscription = RouterInactive.from_product_id(product_id, customer_id)

    return {
        "subscription": subscription,
        "subscription_id": subscription.subscription_id,
    }


def initial_input_form_generator() -> FormGenerator:
    class ImportRouter(FormPage):
        class Config:
            title = "Import Router"

        customer: str
        router_site: str
        hostname: str
        ts_port: int
        router_vendor: RouterVendor
        router_role: RouterRole
        is_ias_connected: Optional[bool] = None
        router_access_via_ts: Optional[bool] = None
        router_lo_ipv4_address: Optional[ipaddress.IPv4Address] = None
        router_lo_ipv6_address: Optional[ipaddress.IPv6Address] = None
        router_lo_iso_address: Optional[str] = None
        router_si_ipv4_network: Optional[ipaddress.IPv4Network] = None
        router_ias_lt_ipv4_network: Optional[ipaddress.IPv4Network] = None
        router_ias_lt_ipv6_network: Optional[ipaddress.IPv6Network] = None

    user_input = yield ImportRouter

    return user_input.dict()


def get_site_by_name(site_name: str) -> S:
    subscription = (
        SubscriptionTable.query.join(
            ProductTable, SubscriptionInstanceTable, SubscriptionInstanceValueTable, ResourceTypeTable
        )
        .filter(SubscriptionInstanceValueTable.value == site_name)
        .filter(ResourceTypeTable.resource_type == "site_name")
        .filter(SubscriptionTable.status == "active")
        .first()
    )
    if not subscription:
        raise ValueError(f"Site with name {site_name} not found")
    return Site.from_subscription(subscription.subscription_id)


@step("Initialize subscription")
def initialize_subscription(
    subscription: RouterInactive,
    hostname: str,
    ts_port: int,
    router_vendor: router_pb.RouterVendor,
    router_site: str,
    router_role: router_pb.RouterRole,
    is_ias_connected: Optional[bool] = None,
    router_lo_ipv4_address: Optional[ipaddress.IPv4Address] = None,
    router_lo_ipv6_address: Optional[ipaddress.IPv6Address] = None,
    router_lo_iso_address: Optional[str] = None,
    router_si_ipv4_network: Optional[ipaddress.IPv4Network] = None,
    router_ias_lt_ipv4_network: Optional[ipaddress.IPv4Network] = None,
    router_ias_lt_ipv6_network: Optional[ipaddress.IPv6Network] = None,
) -> State:
    subscription.router.router_ts_port = ts_port
    subscription.router.router_vendor = router_vendor
    subscription.router.router_site = get_site_by_name(router_site).site
    fqdn = (
        f"{hostname}.{subscription.router.router_site.site_name.lower()}."
        f"{subscription.router.router_site.site_country_code.lower()}"
        ".geant.net"
    )
    subscription.router.router_fqdn = fqdn
    subscription.router.router_role = router_role
    subscription.router.router_access_via_ts = True
    subscription.description = f"Router {fqdn}"
    subscription.router.router_is_ias_connected = is_ias_connected
    subscription.router.router_lo_ipv4_address = router_lo_ipv4_address
    subscription.router.router_lo_ipv6_address = router_lo_ipv6_address
    subscription.router.router_lo_iso_address = router_lo_iso_address
    subscription.router.router_si_ipv4_network = router_si_ipv4_network
    subscription.router.router_ias_lt_ipv4_network = router_ias_lt_ipv4_network
    subscription.router.router_ias_lt_ipv6_network = router_ias_lt_ipv6_network

    subscription = router.RouterProvisioning.from_other_lifecycle(subscription, SubscriptionLifecycle.PROVISIONING)

    return {"subscription": subscription}


@workflow(
    "Import router",
    initial_input_form=initial_input_form_generator,
    target=Target.SYSTEM,
)
def import_router() -> StepList:
    return (
        init
        >> create_subscription
        >> store_process_subscription(Target.CREATE)
        >> initialize_subscription
        >> set_status(SubscriptionLifecycle.ACTIVE)
        >> resync
        >> done
    )
