from uuid import uuid4

from orchestrator.db.models import ProductTable
from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import State, SubscriptionLifecycle, FormGenerator
from orchestrator.workflow import StepList, done, init, step, workflow
from orchestrator.workflows.steps import resync, set_status, store_process_subscription
from pydantic.types import UUID

from gso.products.product_blocks.site import SiteTier
from gso.products.product_types import site
from gso.workflows.site.create_site import initialize_subscription


@step("Create subscription")
def create_subscription() -> State:
    product_id: UUID = ProductTable.query.filter_by(product_type="Site").first().product_id
    subscription = site.SiteInactive.from_product_id(product_id, uuid4())

    return {
        "subscription": subscription,
        "subscription_id": subscription.subscription_id,
    }


def generate_initial_input_form() -> FormGenerator:
    class ImportSite(FormPage):
        class Config:
            title = "Import Site"

        site_name: str
        site_city: str
        site_country: str
        site_country_code: str
        site_latitude: float
        site_longitude: float
        site_bgp_community_id: int
        site_internal_id: int
        site_tier: SiteTier
        site_ts_address: str

    user_input = yield ImportSite
    return user_input.dict()


@workflow(
    "Import Site",
    target=Target.SYSTEM,
    initial_input_form=generate_initial_input_form,
)
def import_site() -> StepList:
    """
    Workflow to import a site without provisioning it.
    """
    return (
            init
            >> create_subscription
            >> store_process_subscription(Target.CREATE)
            >> initialize_subscription
            >> set_status(SubscriptionLifecycle.ACTIVE)
            >> resync
            >> done
    )
